package  conv.convc;

import 	 jam.vector.*;
import 	 jam.operator.*;
import 	rsf.vector.*;

/**
 * These Factories should be static if possible! 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class CcaiFactory {
  Rsf filter;
  
  /**
   * @param filter  Rsf of filter coefficients
   */
  public CcaiFactory(Rsf filter) { this.filter = filter; }
  
  /**
   * This calculates the inputSpace from the outputSpace and filter data,
   * and returns the resulting ccai operator.
   *
   * @param outputSpace the RsfSpace of the output of the ccai operator
   * @return the ccai operator corresponding to filter data and output space
   */
  public Ccai getCcaiFromOutputSpace(RsfSpace outputSpace) {
    RsfSpace filterSpace = (RsfSpace) filter.getSpace();
    RsfSpace inputSpace  =
      CutoffConvolution.createInputSpace(filterSpace, outputSpace);
    return new Ccai(filter, inputSpace, outputSpace, true);
  }
  
  /**
   * This calculates the outputSpace from the InputSpace and filter data,
   * and returns the resulting ccai operator.
   *
   * @param InputSpace the RsfSpace of the Input of the ccai operator
   * @return the ccai operator corresponding to filter data and input space
   */
  public Ccai getCcaiFromInputSpace(RsfSpace inputSpace) {
    RsfSpace filterSpace = (RsfSpace) filter.getSpace();
    RsfSpace outputSpace =
      CutoffConvolution.createOutputSpace(inputSpace, filterSpace);
    return new Ccai(filter, inputSpace, outputSpace, true);
  }
}
