package conv.convi;

import  conv.conviF.InternalConvolutionSpeedTest;
import juice.util.SpeedTestReport;
import  java.io.PrintStream;

public class ComparisonTest {
  public static void main(String[] args) {
    int dim        = 2;
    int filtSize   = 5;
    int numTests   = 10;
    int sampleSize;
    int dataSize;
    PrintStream pstr = System.out;

    long[]   time    = new   long[numTests];
    long[]   timeA   = new   long[numTests];
    long[]   sample  = new   long[numTests];
    long[]   sampleA = new   long[numTests];
    String[] fSizes  = new String[numTests];
    String[] dSizes  = new String[numTests];
    String[] dims    = new String[numTests];

    for (int i = 0; i < numTests; i++) {
      dataSize   = filtSize + i * i * 2;

      double ss = (double) (numTests-i);
      ss *= ss;
      ss *= ss;
      ss *= ss;
      sampleSize = (int) (ss/26e7);

      if (sampleSize < 2) sampleSize = 2;
      SpeedTestReport repA =
	conv.convi.InternalConvolutionSpeedTest.doTest(dim, dataSize,
							filtSize,
							sampleSize);
      SpeedTestReport rep =
	conv.conviF.InternalConvolutionSpeedTest.doTest(dim, dataSize,
						       filtSize,
						       sampleSize);
      printReport(repA);
      printReport(rep);

      time[i]    = rep .getTotalTime();
      timeA[i]   = repA.getTotalTime();
      sample[i]  = rep .getIterations();
      sampleA[i] = repA.getIterations();
      fSizes[i]	 = rep .getProperty("size.filter");
      dSizes[i]	 = rep .getProperty("size.input");
      dims[i]  	 = rep .getProperty("dim");
    }

    printResults(time, timeA, sample, sampleA, fSizes, dSizes, dims, pstr);
  }

  static void printResults(long[] time  , long[] timeA  ,
			   long[] sample, long[] sampleA,
			   String[] fSizes, String[] dSizes, String[] dims,
			   PrintStream pstr) {
    int n = time.length;

    String os   = System.getProperties().getProperty("os.name");
    String suff = null;
    if (os.equals("Linux")) suff = "L";
    if (os.equals( "Irix")) suff = "S";

    pstr.println("% ComparisonTest.java results on " + os + " system");
    
    pstr.print("\n\n% dimension\ndim" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(dims[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% filter sizes\nfsize" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(fSizes[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% data sizes\ndsize" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(dSizes[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% num iterations rsf\niter" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(sample[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% time (msec) rsf\ntottime" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(time[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% time (avg in msec) rsf\navgtime" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(time[i]/((float) sample[i]) + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% num iterations rsfA\niterA" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(sampleA[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% time (msec) rsfA\ntottimeA" + suff + " = [");
    for (int i = 0; i < n; i++)
      pstr.print(timeA[i] + "\t");
    pstr.println("];\n");
    pstr.print("\n\n% time (avg in msec) rsfA\navgtimeA" + suff +
		     " = [");
    for (int i = 0; i < n; i++)
      pstr.print(timeA[i]/((float) sampleA[i]) + "\t");
    pstr.println("];\n");
    pstr.println();
  }

  static void printReport(SpeedTestReport rep) {
    long t 	   = rep.getTotalTime();
    long n 	   = rep.getIterations();
    String osName  = rep.getProperty("os.name");
    String r       = rep.getProperty("rsf.type");
    String dims    = rep.getProperty("dim"        );
    String inpSize = rep.getProperty("size.input" );
    String fltSize = rep.getProperty("size.filter");
    System.err.println("\nResults\n" +
		       "\toperating system      : " + osName  + "\n" +
		       "\tRsf type (rsf or rsfA): " + r + "\n" +
		       "\tDimensions            : " + dims + "\n" +
		       "\tSize(input,filter)    : (" + inpSize + "," +
		       fltSize + ")\n\n" +
		       "\tNumber of iterations  : " + n + "\n" +
		       "\tTime (milli-seconds)  : " + t + "\n" +
		       "\tAverage Time (msec)   : " + t/((float) n));
  }
}
