package  conv.conviF;

import 	 jam.vector.*;
import 	 jam.operator.*;
import   rsf.vector.Axis;
import 	rsfF.vector.*;
import   juice.util.Tester;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class IcafTest {
  public static void main(String[] args) {
    Tester.exit(test(args));
  }

  public static int  test(String[] args) {
    Axis[] ax = new Axis[1];
    ax[0] = new Axis( 4, 2f, 0f, "distance", "m" );

    Axis[] filtAx = new Axis[1];
    filtAx[0] = new Axis( 2, 1f, 0f, "", "" );

    Rsf inRsf = (Rsf) new Rsf(ax);
    inRsf.zero();
    inRsf.d[1] = 1F;

    Rsf filt  = new Rsf(filtAx);
    filt.fill(1F);
    filt.d[1] = -1F;
    
    Icaf myIcaf =
      new IcafFactory(inRsf).getIcafFromFilterSpace((RsfSpace)
						    filt.getSpace());
    Rsf outRsf = (Rsf) myIcaf.image(filt);

    System.out.println( "Input :\n" +  inRsf + "\n\n\n");
    System.out.println( "Filter:\n" +   filt + "\n\n\n");
    System.out.println( "Output:\n" + outRsf + "\n\n");

    boolean passed1 = true;
    passed1 = passed1 && (outRsf.d[0] ==  1F);
    passed1 = passed1 && (outRsf.d[1] == -1F);
    passed1 = passed1 && (outRsf.d[2] ==  0F);

    // 2D test

    Axis[] ax2 = new Axis[2];
    ax2[0] = new Axis( 3, 1f, 0f, "time", "s" );
    ax2[1] = new Axis( 4, 2f, 0f, "distance", "m" );

    Axis[] filtAx2 = new Axis[2];
    filtAx2[0] = new Axis( 2, 1f, 0f, "", "" );
    filtAx2[1] = new Axis( 2, 1f, 0f, "", "" );

    inRsf = (Rsf) new Rsf(ax2);
    inRsf.zero();
    int[] ind = {1,1};
    inRsf.d[inRsf.i(ind)] =  1F;
    filt  = new Rsf(filtAx2);
    filt.fill(1F);
    ind[0] = 1; ind[1] = 0;
    filt .d[ filt.i(ind)] = -1F;
    ind[0] = 0; ind[1] = 1;
    filt .d[ filt.i(ind)] = -1F;

    myIcaf =
      new IcafFactory(inRsf).getIcafFromFilterSpace((RsfSpace)
						    filt.getSpace());
    outRsf = (Rsf) myIcaf.image(filt);

    System.out.println( "Forward Operator (input):\n" +  inRsf + "\n\n\n");
    System.out.println( "Forward Input   (filter):\n" +   filt + "\n\n\n");
    System.out.println( "Forward Output  (output):\n" + outRsf + "\n\n");

    boolean passed2 = true;
    passed2 = passed2 && (outRsf.d[outRsf.i(0,0)] ==  1F);
    passed2 = passed2 && (outRsf.d[outRsf.i(1,0)] == -1F);
    passed2 = passed2 && (outRsf.d[outRsf.i(0,1)] == -1F);
    passed2 = passed2 && (outRsf.d[outRsf.i(1,1)] ==  1F);
    passed2 = passed2 && (outRsf.d[outRsf.i(0,2)] ==  0F);
    passed2 = passed2 && (outRsf.d[outRsf.i(1,2)] ==  0F);

    // Test adjoint in 2D

    filt.zero();
    filt = (Rsf) myIcaf.adjoint().image(outRsf);

    System.out.println( "Adjoint Operator (input):\n" +   inRsf + "\n\n");
    System.out.println( "Adjoint Input   (output):\n" +  outRsf + "\n\n\n");
    System.out.println( "Adjoint Output  (filter):\n" +    filt + "\n\n\n");

    boolean passed3 = true;
    passed3 = passed3 && (filt.d[filt.i(0,0)] ==  1F);
    passed3 = passed3 && (filt.d[filt.i(1,0)] == -1F);
    passed3 = passed3 && (filt.d[filt.i(0,1)] == -1F);
    passed3 = passed3 && (filt.d[filt.i(1,1)] ==  1F);

    // Test in 4D

    int[] inDim = {4,4,4,4};
    inRsf = new Rsf( inDim);
    inRsf.rand();
    int[] filtDim = {2,2,2,2};
    myIcaf =
      new IcafFactory(inRsf).getIcafFromFilterSpace(new RsfSpace(filtDim));
    float diff = rsfF.util.RsfUtil.dotProductTest( myIcaf, "verbose");
    boolean passed4 = ( Math.abs( diff) < .00001);
    return ((passed1 && passed2 && passed3 && passed4) == true ? 0:1);
  }
}
