package conv.conviF;

import  rsfF.vector.*;
import juice.util.Tester;
import juice.util.SpeedTestReport;
import  java.util.Date;
import  java.util.Properties;
import sep.io.ArgumentKeeper;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class InternalConvolutionSpeedTest {
  Rsf                 inpt;
  Rsf                 filt;
  Rsf                 otpt;
  InternalConvolution conv;

  long                time;
  int                 n;

  public InternalConvolutionSpeedTest(int dim,
				      int inputSize,
				      int filterSize) {
    int[] dims = new int[dim];
    for (int i = 0; i < dim; i++)
      dims[i] = inputSize;
    inpt = new Rsf(dims);
    inpt.rand();

    for (int i = 0; i < dim; i++)
      dims[i] = filterSize;
    filt = new Rsf(dims);
    filt.rand();
    
    conv = new InternalConvolution(InternalConvolution.kInput,
				   (RsfSpace) inpt.getSpace(),
				   InternalConvolution.kFilter,
				   (RsfSpace) filt.getSpace());

    otpt = new Rsf((RsfSpace) conv.getRange());
    
    time = 0;
    n    = 0;
  }

  public void doOnce() {
    long startTime = new Date().getTime();

    conv.imageBinary(inpt, filt, otpt);

    time += new Date().getTime() - startTime;
    n++;
  }

  public int    getNumber() { return n; }
  public long   getTime  () { return time; }

  public static SpeedTestReport doTest(int dim, int inputSize,
				       int filterSize, int numIterations) {
    System.gc();

    InternalConvolutionSpeedTest
      test = new InternalConvolutionSpeedTest(dim, inputSize, filterSize);

    System.out.println("InternalConvolutionSpeedTest");
    for (int i = 0; i < numIterations; i++ ) {
      test.doOnce();
      System.out.println("Done with convolution #" + (i+1) + " of " +
			 numIterations);
    }
    SpeedTestReport
      report = new SpeedTestReport("InternalConvolutionSpeedTest",
				   test.getTime(), test.getNumber());
    Properties p = report.getProperties();
    p.put("rsfF.type", test.inpt.getClass().getName());
    p.put("dim"        , new Integer(       dim).toString());
    p.put("size.input" , new Integer( inputSize).toString());
    p.put("size.filter", new Integer(filterSize).toString());
    return report;
  }

  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    int	  dim        = argKeeper.getInt("dim"       ,  "6");
    int   inputSize  = argKeeper.getInt("inputsize" ,  "4");
    int   filterSize = argKeeper.getInt("filtersize",  "3");
    int   niter      = argKeeper.getInt("niter"     ,  "2");

    SpeedTestReport 
      report = doTest(dim, inputSize, filterSize, niter);
    long t = report.getTotalTime();
    long n = report.getIterations();
    String osName = report.getProperty("os.name");
    String r      = report.getProperty("rsfF.type");
    String dims    = report.getProperty("dim"        );
    String inpSize = report.getProperty("size.input" );
    String fltSize = report.getProperty("size.filter");
    System.out.println("\nResults\n" +
		       "\toperating system      : " + osName  + "\n" +
		       "\tRsf type (rsf or rsfA): " + r + "\n" +
		       "\tDimensions            : " + dims + "\n" +
		       "\tSize(input,filter)    : (" + inpSize + "," +
		       fltSize + ")\n\n" +
		       "\tNumber of iterations  : " + n + "\n" +
		       "\tTime (milli-seconds)  : " + t + "\n" +
		       "\tAverage Time (msec)   : " + t/((float) n));
  }
}
