package  conv.convt;

import 	 jam.vector.*;
import 	 jam.operator.*;
import 	rsf.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class TcafFactory {
  Rsf input;
  
  /**
   * @param input  Rsf of input data
   */
  public TcafFactory(Rsf input) { this.input = input; }
  
  /**
   * This calculates the filterSpace from the outputSpace and input data,
   * and returns the resulting tcaf operator.
   *
   * @param outputSpace the RsfSpace of the output of the tcaf operator
   * @return the tcaf operator corresponding to the input data and output space
   */
  public Tcaf getTcafFromOutputSpace(RsfSpace outputSpace) {
    RsfSpace inputSpace  = (RsfSpace) input.getSpace();
    RsfSpace filterSpace =
      TransientConvolution.createFilterSpace(inputSpace, outputSpace);
    return new Tcaf(input, filterSpace, outputSpace, true);
  }
  
  /**
   * This calculates the outputSpace from the filterSpace and input data,
   * and returns the resulting tcaf operator.
   *
   * @param filterSpace the RsfSpace of the filter of the tcaf operator
   * @return the tcaf operator corresponding to the input data and filter space
   */
  public Tcaf getTcafFromFilterSpace(RsfSpace filterSpace) {
    RsfSpace inputSpace  = (RsfSpace) input.getSpace();
    RsfSpace outputSpace =
      TransientConvolution.createOutputSpace(inputSpace, filterSpace);
    return new Tcaf(input, filterSpace, outputSpace, true);
  }
}
