package  conv.convt;

import 	 jam.vector.*;
import 	 jam.operator.*;
import 	rsf.vector.*;
import   rsf.vector.Axis;
import juice.util.Tester;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class TcaiTest {
  public static void main(String[] args) {
    Tester.exit(test(args));
  }
  
  public static int  test(String[] args) {
    Axis[] ax = new Axis[1];
    ax[0] = new Axis(4, 2f, 0f, "distance", "m");
    
    Axis[] filtAx = new Axis[1];
    filtAx[0] = new Axis(2, 1f, 0f, "", "");
    
    Rsf inRsf = RsfFactory.newRsf(ax);
    inRsf.zero();
    ((Rsf1) inRsf).setDatum(1,  1f);
    
    Rsf filt  = RsfFactory.newRsf(filtAx);
    filt .fill(1f);
    ((Rsf1) filt) .setDatum(1, -1f);
    
    Tcai myTcai =
      new TcaiFactory(filt).getTcaiFromInputSpace((RsfSpace)
						  inRsf.getSpace());
    Rsf outRsf = (Rsf) myTcai.image(inRsf);
    
    System.out.println("Input :\n" +  inRsf + "\n\n\n");
    System.out.println("Filter:\n" +   filt + "\n\n\n");
    System.out.println("Output:\n" + outRsf + "\n\n");
    
    Rsf ideal = RsfFactory.newRsf((RsfSpace) outRsf.getSpace());
    ideal.zero();
    ((Rsf1) ideal).setDatum(1,  1f);
    ((Rsf1) ideal).setDatum(2, -1f);
    
    boolean passed1 = outRsf.equals(ideal);
    
    // 2D test
    
    Axis[] ax2 = new Axis[2];
    ax2[0] = new Axis(3, 1f, 0f, "time", "s");
    ax2[1] = new Axis(4, 2f, 0f, "distance", "m");
    
    Axis[] filtAx2 = new Axis[2];
    filtAx2[0] = new Axis(2, 1f, 0f, "", "");
    filtAx2[1] = new Axis(2, 1f, 0f, "", "");
    
    inRsf = RsfFactory.newRsf(ax2);
    inRsf.zero();
    ((Rsf2) inRsf).setDatum(1,1,  1f);
    
    filt  = RsfFactory.newRsf(filtAx2);
    ((Rsf2) filt). setDatum(0,0,  1f); ((Rsf2) filt).setDatum(0,1, -1f);    
    ((Rsf2) filt). setDatum(1,0, -1f); ((Rsf2) filt).setDatum(1,1,  1f);
    
    myTcai =
      new TcaiFactory(filt).getTcaiFromInputSpace((RsfSpace)
						  inRsf.getSpace());
    outRsf = (Rsf) myTcai.image(inRsf);
    
    System.out.println("Forward Operator (input):\n" +  inRsf + "\n\n\n");
    System.out.println("Forward Input   (filter):\n" +   filt + "\n\n\n");
    System.out.println("Forward Output  (output):\n" + outRsf + "\n\n");
    
    ideal = RsfFactory.newRsf((RsfSpace) outRsf.getSpace());
    ideal.fill(0F);
    ((Rsf2) ideal).setDatum(1,1,  1F);
    ((Rsf2) ideal).setDatum(2,2,  1F);
    ((Rsf2) ideal).setDatum(1,2, -1F);
    ((Rsf2) ideal).setDatum(2,1, -1F);
    
    boolean passed2 = outRsf.equals(ideal);
    
    // Test adjoint in 2D
    
    inRsf .zero();
    outRsf.zero();
    ((Rsf2) outRsf).setDatum(0,1, 1f);
    inRsf = (Rsf) myTcai.adjoint().image(outRsf);
    
    System.out.println("Adjoint Input :\n" +  outRsf + "\n\n\n");
    System.out.println("Adjoint Filter:\n" +    filt + "\n\n\n");
    System.out.println("Adjoint Output:\n" +   inRsf + "\n\n");
    
    ideal = RsfFactory.newRsf((RsfSpace) inRsf.getSpace());
    ideal.zero();
    ((Rsf2) ideal).setDatum(0,0, -1f);
    ((Rsf2) ideal).setDatum(0,1,  1f);
    
    boolean passed3 = inRsf.equals(ideal);
    
    // Test in 4D
    
    int[] inDim   = {4,4,4,4};
    int[] filtDim = {2,2,2,2};
    filt  = RsfFactory.newRsf(filtDim);
    filt.rand();
    myTcai = new TcaiFactory(filt).getTcaiFromInputSpace(new RsfSpace(inDim));
    float diff = rsf.util.RsfUtil.dotProductTest(myTcai, "verbose");
    boolean passed4 = (Math.abs(diff) < .00001);
    
    return ((passed1 && passed2 && passed3 && passed4) == true ? 0:1);
  }
}
