package conv.deconF;

import rsfF.vector.*;
//import rsfF.operator.*;
import sep.io.*;
import jam.vector.*;
import jam.operator.*;
import juice.solver.*;
import juice.operator.*;
import conv.conviF.*; 

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */

/**
* I solve d = M b for b. 
* M is a    known convolution operator of the time series m (reflectiviy). 
* b is an unknown filter (filter). 
* d is a    known time series (seismic trace). 
* 
* I create the normal equations: d_L = M'd and L = M'M and 
* call a CGSolver to solve M'd = M'M b or d_L = L b. 
*/ 
public class IcafDecon { 
  //private Rsf m; 
  //private Rsf d;
  private int niter; 

  public static void main(String[] args) {

    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    int niter =    argKeeper.getInt("niter", "15");

    // Get d (the trace)
    Rsf trace = new Rsf(System.in); 

    // Get the reflectivity that later builds the convolution operator
    Rsf rflct = new Rsf(argKeeper.getString( "reflectivity"));

    IcafDecon myDecon = new IcafDecon(niter); 
    Rsf wavlt         = myDecon.solve(rflct,trace); 

    wavlt.write(System.out); 
  }

  /** 
   * IcafDecon is a bit like a solver. I use a similar interface design. 
   * I would like to have 
   *   IcafDecon(Rsf m, Stopper)).solve(       Rsf d, Rsf b) (Decon defined   )
   *   IcafDecon(       Stopper)).solve(Rsf m, Rsf d, Rsf b) (just like solver)
   * Which one should I prefer? 
   */
  public IcafDecon(int niter) { 
    this.niter= niter;
  }

  /**
   *just like operator's image(b)     and b=image() 
   *I could have         solve(m,d,b) and b=solve(m,b)
   *where do I put the constructor
   */
  public void solve(Rsf m, Rsf d, Rsf b){ 
    b = this.solve(m,d);
  }

  //I need this extra routine since constructors can and should not return
  //anything. Maybe a better name. 
  public Rsf solve(Rsf m, Rsf d){ 
    //this.d = d;
    //this.m = m; 

    // Create the Icaf operator 
    hasAdjoint mOp = new IcafFactory(m).getIcafFromOutputSpace(
	           (RsfSpace) d.getSpace()); 
    // Create the positive definite equation M'd = M'M b
    Vector md = mOp.adjoint().image(d);
    CompoundLinearOperatorAdjoint
      mmOp = new CompoundLinOpAdj(mOp, mOp.adjoint());
    // Create an initial b (the wavelet) 
    Rsf b = (Rsf) mmOp.getDomain().newMember();
    b.zero(); //b.rand();

    // Solve
    // new CGSolver(new IteratorSmall()).solve(mmOp, md, b);
    new    CGSolver(                   ).solve(mmOp, md, b);

    return b; 
  }
}


    

    
