package conv.deconF;

import rsfF.vector.*;
// import rsfF.operator.*;
import sep.io.*;
import jam.vector.*;
import jam.operator.*;
import juice.solver.*;
import juice.operator.*;
import conv.conviF.*; 

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */

/**
* I solve d = B m for m. 
* B is a known convolution operator of the filter b (wavelet).
* m is a unknown reflectivity series.
* d is a known time series (seismic trace). 
*
* I create the normal equations: Bd = B'd and BB = B'B and 
* call a CGSolver to solve B'd = B'B m or Bd = BB m 
*/ 
public class IcaiDecon { 
  //  private Rsf d; 
  //  private Rsf b;
  private int niter; 

  public static void main(String[] args) {

    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    int   niter    = argKeeper.getInt  ("niter", "1500");

    // Get d (the trace)
    Rsf trace   = new Rsf(System.in); 

    // Get the filter that later builds the convolution operator
    Rsf wavelet = new Rsf(argKeeper.getString( "wavelet"));

    IcaiDecon myDecon = new IcaiDecon(niter) ; 
    Rsf reflectivity = myDecon.solve(wavelet,trace); 

    reflectivity.write(System.out); 
  }

  public IcaiDecon(int niter) { 
    this.niter = niter;
  }

  public void solve(Rsf b, Rsf d, Rsf m){ 
    m = this.solve(b,d);
  }

  public Rsf solve(Rsf b, Rsf d){ 

    // Create the Icai operator 
    hasAdjoint bOp = new IcaiFactory(b).getIcaiFromOutputSpace(
		  (RsfSpace) d.getSpace()); 

    // Create the positive definite equation B'd = B'B m

    Vector bd = bOp.adjoint().image(d);
    CompoundLinearOperatorAdjoint
      bbOp = new CompoundLinOpAdj(bOp, bOp.adjoint());

    // Create an initial m (the reflectivity series) 
    Rsf m = (Rsf) bbOp.getDomain().newMember();
    m.zero(); //m.rand();

    // Solve
    new   CGSolver(                   ).solve(bbOp, bd, m);
    //new CGSolver(new IteratorSmall()).solve(bbOp, bd, m);

    return m;
  }
}


    

    
